
#include "CelestialObject.h"


CelestialObject::CelestialObject(string name, Time *rightAscension, Degrees90 *declination, string magnitude, Observer *observer) {
    this->rightAscension = rightAscension;
    this->declination = declination;
    this->observer = observer;
    this->name = name;
    this->recalculateHorizontalCoordinates();
    this->magnitude = magnitude;

    // DEBUG
    cout << "Right Ascension: " << rightAscension->getC() << " :: " << rightAscension->getHour() << "h " << rightAscension->getMinutes() << "m " << rightAscension->getSeconds() << "s" << endl;
    // We should print a '-' in front of degrees if it is a negative zero (because they cannot be represented by C++ integers)
    if (declination->getNegativeZero()) {
        cout << "Declination: " << declination->getC() << " :: -" << declination->getHour() << "º " << declination->getMinutes() << "' " << declination->getSeconds() << "\"" << endl;
    } else {
        cout << "Declination: " << declination->getC() << " :: " << declination->getHour() << "º " << declination->getMinutes() << "' " << declination->getSeconds() << "\"" << endl;
    }
    cout << "Altitude: " << altitude->getC() << " :: " << altitude->getHour() << "º " << altitude->getMinutes() << "' " << altitude->getSeconds() << "\"" << endl;
    cout << "European azimuth (South=0º): " << azimuth->getC() << " :: " << azimuth->getHour() << "º " << azimuth->getMinutes() << "' " << azimuth->getSeconds() << "\"" << endl;
    cout << "American azimuth (North=0º): " << americanAzimuth->getC() << " :: " << americanAzimuth->getHour() << "º " << americanAzimuth->getMinutes() << "' " << americanAzimuth->getSeconds() << "\"" << endl;
    cout << "Culmination (Universal Time): " << timeOfCulmination->getC() << " :: " << timeOfCulmination->getHour() << "h " << timeOfCulmination->getMinutes() << "m " << timeOfCulmination->getSeconds() << "s" << endl;
}

void CelestialObject::recalculateHorizontalCoordinates() {
    //// Calculate Horizontal coordinates from Equatorial coordinates
    // Get Hour Angle
    float H = this->observer->getLocalSiderealTime()->getC() - this->rightAscension->getC(); // H = O - alpha
    if (H < 0) {
        H = 24 + H;
    }

    this->hourAngle = new Time(H);

    // Get H in radians
    H = this->hourAngle->getCInRadians();

    // Get Declination in radians
    float DC = this->declination->getCInRadians();

    // Get observer's Latitude in radians
    float LAT = this->observer->getLatitude()->getCInRadians();

    // Get altitude with repect to the observer
    // * With the following formula we get an 'h' without ambiguity
    float h = asin((sin(DC) * sin(LAT)) + (cos(H) * cos(DC) * cos(LAT)));

    /// Get azimuth with repect to the observer
    // European system: Azimuth 0 starts at South
    float aFromSin = (cos(DC) * sin(H)) / cos(h);
    float aFromCos = (sin(LAT)*cos(DC)*cos(H)-sin(DC)*cos(LAT))/cos(h);
    // Atan2 is used to avoid ambiguity of a
    float a = atan2(aFromSin, aFromCos); // European azimuth
    if (a < 0) {
        a = 2*PI + a; // Make azimuth always positive, just for clarity
    }

    // American system: Azimuth 0 starts at North
	// Method from Kstars: skypoint.cpp
	float americanA;
	float arg = (sin(DC) - sin(LAT)*sin(h)) / (cos(LAT)*cos(h));
	if ( arg <= -1.0 ) {
	    americanA = PI;
    } else if ( arg >= 1.0 ) {
        americanA = 0.0;
    } else {
        americanA = acos( arg );
    }
    // resolve acos() ambiguity
    if ( sin(H) > 0.0 ) americanA = 2.0*PI - americanA; // American azimuth

	// Convert to degrees
    h = h * (180/PI);
    a = a * (180/PI);
    americanA = americanA * (180/PI);


    this->altitude = new Degrees90(h);
    this->azimuth = new Degrees360(a);
    this->americanAzimuth = new Degrees360(americanA);

    //// Calculate Time of culmination
    float t = (this->rightAscension->getC() - this->observer->getSiderealTime()->getC() - this->observer->getLongitude()->getC() ) / 1.00273791;
    if (t < 0) {    // Adjust
        t = 24 + t;
    }

    this->timeOfCulmination = new Time(t);
}

Degrees360 *CelestialObject::getAzimuth() {
    return (azimuth);
}

Degrees90 *CelestialObject::getAltitude() {
    return (altitude);
}

Time *CelestialObject::getRightAscension() {
    return (rightAscension);
}

Degrees90 *CelestialObject::getDeclination() {
    return (declination);
}

Time *CelestialObject::getTimeOfCulmination() {
    return timeOfCulmination;
}

string CelestialObject::getName() {
    return this->name;
}

string CelestialObject::getMagnitude() {
    return this->magnitude;
}
